#include "CanvasView.h"
#include "Canvas.h"
#include "base_gui/gMiddle.h"
#include <iostream>
using namespace std;

CanvasView::CanvasView( QWidget *parent, const char *name ): 
	QCanvasView( parent, name ), particleVelocity(2),
	delay(-1), cycleCount(0)
{
	oCanvas = new Canvas(this, "Canvas");
	connect( oCanvas, SIGNAL( sig_advanced() ), this, SLOT( advanced() ) );
	
	oCanvas->setDoubleBuffering(true);
	this->setCanvas(oCanvas);
	
	oCanvas -> setBackgroundColor (Qt::darkGray);
	oCanvas->resize(this->width()-2, this->height()-2);
}

CanvasView::~CanvasView()
{
    hide();
}

void CanvasView::setGeometry(int x1, int x2, int x3, int x4)
{
    QWidget::setGeometry(x1, x2, x3, x4);
    oCanvas->resize(this->width()-2,this->height()-2);
}

void CanvasView::setGeometry(QRect rect)
{
    QWidget::setGeometry(rect);
    oCanvas->resize(this->width()-2,this->height()-2);
}

void CanvasView::position()
{
    list<gBelt*>::iterator iter;
    int y_offset = 0;
    for ( iter = beltList.begin(); iter != beltList.end(); iter++ )
    {
	gBelt *belt = (*iter);
	belt->setX(60);
	belt->setY(y_offset + gEngine::HEIGHT + gAlpha::V_GAP);
	belt->setVelocity(belt->direction * particleVelocity, 0);
	belt->setAnimated(true);
	belt->show();
	
	//friend access
	int x_offset = 0;
	list<Device*>::iterator iter2;
	for ( iter2 = belt->devices.begin(); iter2 != belt->devices.end(); iter2++ )
	{
	    gMiddle *device = (gMiddle*) (*iter2);
	    
	    if (belt->direction > 0) {
		device->setX( belt->x() + belt->WIDTH - gMiddle::WIDTH - x_offset );
	    } else {
		device -> setX ( belt->x() + gMiddle::WIDTH + x_offset);
	    }
	    device->setY( belt->y() + gMiddle::HEIGHT);
	    device->setVelocity(particleVelocity, 0);
	    device->setAnimated(true);
	    device->show();
	    
	    x_offset += gMiddle::WIDTH + 4*gMiddle::H_GAP;
	    
	}
	
	y_offset += 4*gEngine::HEIGHT + gMiddle::HEIGHT;
    }
    
    gBelt *first = (*beltList.begin());
    iter = beltList.end();
    iter--;
    gBelt *last = (*iter);
    
    gAlpha *alpha = new gAlpha(oCanvas, first);
    if (first->direction > 0) {
	alpha->setX( first->x() - gAlpha::WIDTH );
    } else {
	alpha->setX( first->x() + first->WIDTH );
    }
    alpha->setY( first->y() - gEngine::HEIGHT - gAlpha::V_GAP );
    alpha->setVelocity(particleVelocity,0);
    alpha->setAnimated(true);
    alpha -> show();
    deviceList.push_front(alpha);
    
    gOmega *omega = new gOmega(oCanvas, last);
    if (last->direction > 0) {
	omega->setX( last->x() + last->WIDTH );
    } else {
	omega->setX( last->x() - gOmega::WIDTH );	
    }
    omega -> setY ( last->y() - gEngine::HEIGHT - gAlpha::V_GAP );
    omega -> setVelocity(particleVelocity,0);
    omega -> setAnimated(true);
    omega -> show();
    deviceList.push_front(omega);
    connect( omega, SIGNAL( sig_stats(int, int, int, int, int, int) ), this, SIGNAL( sig_stats(int, int, int, int, int, int) ) );
    

    
}

void CanvasView::create()
{
    gBelt *belt1 = new gBelt(oCanvas, 3, 1);
    beltList.push_back(belt1);
    
    gBelt *belt2 = new gBelt(oCanvas, 4, -1, belt1);
    beltList.push_back(belt2);
    
    gBelt *belt3 = new gBelt(oCanvas, 4, -1, belt2);
    beltList.push_back(belt3);
    
    gBelt *belt4 = new gBelt(oCanvas, 2, 1, belt3);
    beltList.push_back(belt4);
    
    gBelt *belt5 = new gBelt(oCanvas, 5, 1, belt4);
    beltList.push_back(belt5);
    
    gMiddle *mid = new gMiddle(oCanvas, belt1, belt2, 0);
    
    gMiddle *mid2 = new gMiddle(oCanvas, belt1, belt2, 0);
    
    gMiddle *mid3 = new gMiddle(oCanvas, belt2, belt3, 1);
    
    gMiddle *mid32 = new gMiddle(oCanvas, belt2, belt3, 1);
    
    gMiddle *mid33 = new gMiddle(oCanvas, belt2, belt3, 1);
    
    gMiddle *mid34 = new gMiddle(oCanvas, belt2, belt3, 1);
    
    gMiddle *mid4 = new gMiddle(oCanvas, belt3, belt4, 2);
    
    gMiddle *mid5 = new gMiddle(oCanvas, belt4, belt5, 3);
    
    gMiddle *mid52 = new gMiddle(oCanvas, belt4, belt5, 3);
    
    position();
    
    oCanvas->update();
}

void CanvasView::start(int delay)
{
    oCanvas -> setAdvancePeriod(delay);
    this->delay = delay;
    
    oCanvas->advance();
}

void CanvasView::changeDelay(int delay)
{
    oCanvas -> setAdvancePeriod(delay);
    this->delay = delay;
}

void CanvasView::changeSpeed(int speed)
{
    oCanvas -> setAdvancePeriod(-1);
    double difference = speed - particleVelocity;
    particleVelocity = speed;
    QCanvasItemList list = oCanvas->allItems();
    QCanvasItemList::iterator iter;
    for ( iter = list.begin(); iter != list.end(); iter++ ) {
	(*iter)->setVelocity( (*iter)->xVelocity() + difference, (*iter)->yVelocity() );
    }
    changeMiddleSpeed(speed);
    oCanvas->update();
    oCanvas -> setAdvancePeriod(delay);
}

void CanvasView::changeMiddleSpeed(int speed)
{
    list<gBelt*>::iterator iter;
    for ( iter = beltList.begin(); iter != beltList.end(); iter++ )
    {
    	list<Device*>::iterator iter2;
	for ( iter2 = (*iter)->devices.begin(); iter2 != (*iter)->devices.end(); iter2++)
	{
		gMiddle *dev = (gMiddle*) (*iter2);
		if (dev->stage == gMiddle::GET_ENGINE) {
			dev->setArrivalCoords();
		} else if (dev->stage == gMiddle::PUT_ENGINE) {
			dev->setTargetCoords();
		}
	}
    }
    
}

void CanvasView::stop()
{
    oCanvas->setAdvancePeriod(-1);
    delay = -1;
}

void CanvasView::advanced()
{
    
    cycleCount++;
    if ( cycleCount % ((int) ((gEngine::WIDTH + gBelt::GAP_SIZE) / particleVelocity) ) == 0 ) {
	cycleCount = 0;
	list<Device*>::iterator iter;
	for ( iter = deviceList.begin(); iter != deviceList.end(); iter++)
	{
	    (*iter)->doStep();
	}
	list<gBelt*>::iterator iter2;
	for ( iter2 = beltList.begin(); iter2 != beltList.end(); iter2++)
	{
	    (*iter2)->doStep();
	}
    }
}
