#include "gEngine.h"
#include "gAlpha.h"

const QColor gAlpha::C_DEVICE = darkCyan;
const QColor gAlpha::C_INSIDE = gray;

const int gAlpha::H_GAP = 10;
const int gAlpha::V_GAP = 10;
const int gAlpha::WIDTH = gEngine::WIDTH + 2*H_GAP;
const int gAlpha::HEIGHT = gEngine::HEIGHT + 2*V_GAP;

gAlpha::gAlpha(QCanvas *canvas, gBelt *toBelt):
	QCanvasPolygonalItem(canvas), Alpha(toBelt), cycleCount(0), lastCreated(0)
{
    setZ(5);
}

gAlpha::~gAlpha()
{
    hide();
}

void gAlpha::doStep()
{
    gEngine *newEng = new gEngine(canvas());
    newEng -> setX(x() + gAlpha::H_GAP);
    newEng -> setY(y() + gAlpha::V_GAP);
    newEng -> setVelocity ( ((gBelt*) toBelt)->direction * xVelocity(), 0);
    newEng -> setAnimated(true);
    newEng -> show();
    
    cycleCount = 0;
    lastCreated = newEng;
}

void gAlpha::drawShape ( QPainter & p )
{
    QWMatrix original = p.worldMatrix();
    p.translate(x(), y());
    //p.setPen( NoPen );
    p.setBrush(C_DEVICE);
    p.drawRect(0, 0, WIDTH, gAlpha::HEIGHT);
    
    p.setBrush(C_INSIDE);
    p.drawRect(0, V_GAP, WIDTH, HEIGHT-2*V_GAP);
    
    p.setWorldMatrix(original);
}

QPointArray gAlpha::areaPoints() const
{
    int dx = (int) x();
    int dy = (int) y();
    QPointArray pa(4);
    pa.putPoints(0, 4, 	0, 0, 	WIDTH, 0, 	WIDTH,HEIGHT,	0, HEIGHT );
    pa.translate(dx,dy);
    return pa;
}

void gAlpha::advance(int phase)
{
    if (phase == 1) {
	if (!lastCreated) return;
	cycleCount++;
	if (cycleCount % ( (int) ((gEngine::WIDTH + H_GAP + gBelt::HEIGHT) / xVelocity()) ) == 0) {
	    cycleCount = 0;
	    toBelt->putEngine(lastCreated);
	    lastCreated = 0;
	}
    }
}
