#include "gBelt.h"
#include <qpainter.h>
#include <iostream>
using namespace std;

const QColor gBelt::C_OUTLINE = black;
const QColor gBelt::C_RECT = darkCyan;
const QColor gBelt::C_WHEELS = gray;
const int gBelt::HEIGHT = 20;
const int gBelt::GAP_SIZE = gEngine::WIDTH;

gBelt::gBelt( QCanvas *canvas, int capacity, int direction, Belt *previous ) :
	QCanvasPolygonalItem( canvas ), Belt(capacity, previous), angle(0), 
	WIDTH( (capacity + 1) * GAP_SIZE + capacity * gEngine::WIDTH ),
	direction(direction)
{
}

gBelt::~gBelt()
{
    hide();
}

gEngine *gBelt::getEngineAtEnd()
{
    if (engines.empty()) return 0;
    if (engines.size() != capacity) return 0;
    Engine *to_return = engines.back();
    engines.pop_back();
    gEngine *cast = (gEngine*) to_return;
    return cast;
}

int gBelt::deviceCount() const
{
    return devices.size();
}

void gBelt::drawShape ( QPainter & p )
{
    //p.setRenderHint(QPainter::Antialiasing, true);
    QWMatrix original = p.worldMatrix();
    QPen newPen( C_OUTLINE, 3 );
    p.translate(x(), y());
    
    p.setPen( newPen );
    
    //main rectangle
    p.setBrush( C_RECT );
    p.drawRect(HEIGHT/2, 0, WIDTH-HEIGHT, HEIGHT);
    
    //wheels
    p.setBrush( C_WHEELS );
    p.drawEllipse(0, 0, HEIGHT, HEIGHT);
    p.drawEllipse(WIDTH - HEIGHT, 0, HEIGHT, HEIGHT);
    
    //axis
    //translate to center
    QWMatrix preCenter = p.worldMatrix();
    p.translate(HEIGHT/2, HEIGHT/2);
    p.rotate(angle);
    
    //1st wheel axis
    //p.drawLine(-HEIGHT/2, 0, HEIGHT/2, 0);
    //p.drawLine(0, -HEIGHT/2, 0, HEIGHT/2);
    p.drawLine(0, -HEIGHT/2+1, 0, 0);

    p.setWorldMatrix(preCenter);
    p.translate(WIDTH-HEIGHT/2, HEIGHT/2);
    p.rotate(angle);
    
    //2nd wheel axis
    //p.drawLine(-HEIGHT/2, 0, HEIGHT/2, 0);
    //p.drawLine(0, -HEIGHT/2, 0, HEIGHT/2);
    p.drawLine(0, -HEIGHT/2+1, 0, 0);
  
    p.setWorldMatrix(original);
}

QPointArray gBelt::areaPoints() const
{
    int dx = (int) x();
    int dy = (int) y();
    QPointArray pa(4);
    pa.putPoints(0, 4, 	0, 0, 	WIDTH, 0, 	WIDTH,HEIGHT,	0, HEIGHT );
    pa.translate(dx,dy);
    return pa;
}

void gBelt::advance(int phase)
{
    if (phase == 1) {
	angle += (int) xVelocity()*10;
	update();
    }
}

void gBelt::doStep()
{
    if (!devices.empty()) {
	if (lastUsed == devices.end()) lastUsed = devices.begin();	//every time use another device to transfer engines
	(*lastUsed)->doStep();
	lastUsed++;
    }
}

const QPoint* gBelt::getArrivalPoint() const
{
    QPoint *p;
    if (direction > 0) {
	p = new QPoint( (int) x() + gBelt::HEIGHT, (int) y() );
    } else {
	p = new QPoint( (int) x() + WIDTH - gBelt::HEIGHT - gEngine::WIDTH,  (int) y() );
    }
    return p;
}
