#include "gEngine.h"
#include <qpainter.h>
#include <iostream>
using namespace std;

const QColor gEngine::C_ENGINE = yellow;
const QColor gEngine::C_FULL = green;
const QColor gEngine::C_EMPTY = darkGray;
const QColor gEngine::C_BAD = red;
const int gEngine::PART_SIZE = 5;
const int gEngine::GAP_SIZE = 4;

const int gEngine::WIDTH = PART_SIZE * 4 + GAP_SIZE * 5;
const int gEngine::HEIGHT = (int) (WIDTH * 0.6);
const int gEngine::PART_HEIGHT = (int) (HEIGHT*0.6);

gEngine::gEngine( QCanvas *canvas ):
	QCanvasPolygonalItem(canvas), stepsTaken(0)
{
}

gEngine::~gEngine()
{
    hide();
}

void gEngine::drawShape ( QPainter & p )
{
    p.translate(x(), y());
    p.setBrush( C_ENGINE );
//    p.setPen( C_ENGINE );
    p.drawRect(0, 0, WIDTH, HEIGHT);
    
    QColor *colors[4];
    
    //determine part colors
    colors[0] = (carcass) ? new QColor(C_FULL) : new QColor(C_BAD);
    colors[1] = (valves) ? new QColor(C_FULL) : new QColor(C_BAD);
    colors[2] = (starter) ? new QColor(C_FULL) : new QColor(C_BAD);
    colors[3] = (lid) ? new QColor(C_FULL) : new QColor(C_BAD);
    
    for (int i = 0; i < stepsTaken; i++) {	//paint full parts
	p.setBrush(*colors[i]);
	p.drawRect(GAP_SIZE*(i+1) + PART_SIZE*i, 0, PART_SIZE, PART_HEIGHT);
    }
    for (int i = stepsTaken; i < 4; i++) {	//paint empty parts
	p.setBrush(C_EMPTY);
	p.drawRect(GAP_SIZE*(i+1) + PART_SIZE*i, 0, PART_SIZE, PART_HEIGHT);
    }
    //delete[] colors;
    p.translate(-x(), -y());
}

QPointArray gEngine::areaPoints() const
{
    int dx = (int) x();
    int dy = (int) y();
    QPointArray pa(4);
    pa.putPoints(0, 4, 	0, 0, 	WIDTH, 0, 	WIDTH,HEIGHT,	0, HEIGHT );
    pa.translate(dx,dy);
    return pa;
}
