#include "gEngine.h"
#include "gMiddle.h"
#include <iostream>
using namespace std;

const QColor gMiddle::C_DEVICE = darkCyan;
const QColor gMiddle::C_INSIDE = gray;

const int gMiddle::H_GAP = 10;
const int gMiddle::V_GAP = 5;
const int gMiddle::WIDTH = gEngine::WIDTH + 2*H_GAP;
const int gMiddle::HEIGHT = gEngine::HEIGHT + 2*V_GAP;

gMiddle::gMiddle(QCanvas *canvas, gBelt *fromBelt, gBelt *toBelt, int whatToDo):
	QCanvasPolygonalItem(canvas), Middle(fromBelt, toBelt, whatToDo), defect(false), 
	stage(NONE), cycleCount(0), trigger(0)
	
{
    setZ(11);
}

gMiddle::~gMiddle()
{
    hide();
}

void gMiddle::doStep()
{
    if (stage != NONE) return;
    holding = ((gBelt*) fromBelt)->getEngineAtEnd();
    if (holding) {
	holding->setZ(10);
	stage = GET_ENGINE;
	cycleCount = 0;
	screwed = false;
	
	trigger = (int) ( (gEngine::WIDTH + gBelt::GAP_SIZE) * ((gBelt*) fromBelt)->deviceCount() / (4*xVelocity()) );
	
	setArrivalCoords();
	
	QPen newPen ( white, 2, DotLine );
	line1 = new QCanvasLine(canvas());
	line2 = new QCanvasLine(canvas());
	line1->setPen(newPen);
	line2->setPen(newPen);
	line1->show();
	line2->show();
	line1->setZ(12);
	line2->setZ(12);
		
    }
}

void gMiddle::drawShape ( QPainter & p )
{
    QWMatrix original = p.worldMatrix();
    p.translate(x(), y());
    //p.setPen( NoPen );
    p.setBrush(C_DEVICE);
    p.drawRect(0, 0, WIDTH, gMiddle::HEIGHT);
    
    int offset = (int) (H_GAP + gEngine::PART_SIZE*whatToDo + gEngine::GAP_SIZE*(whatToDo+1) );
    offset += gEngine::PART_SIZE/2;	//center
      
    QWMatrix preCenter = p.worldMatrix();
    p.translate(offset, HEIGHT - gEngine::PART_HEIGHT*5/3);
    //inside
    p.setBrush( C_INSIDE );
    p.drawRect( -gEngine::PART_SIZE/2, 0, gEngine::PART_SIZE, gEngine::PART_HEIGHT*5/3 );
    
    
    
    if (stage == MAKE_PART) {
	
	if (!screwed) {
	    //puts part into engine during this time interval
	    if ( (cycleCount % trigger) <= (trigger / 2) ) {
		p.translate( 0, ( gEngine::PART_HEIGHT + V_GAP ) * ( (cycleCount % trigger) ) * 2 / trigger);
		p.setBrush( *part_color );
		//part
		p.drawRect( -gEngine::PART_SIZE/2, gEngine::PART_HEIGHT*2/3-1, gEngine::PART_SIZE, gEngine::PART_HEIGHT);
	    }
	
	    if ( (cycleCount % trigger) == (trigger / 2) ) {
		screwPart();
	    }
	}
	
	if ( (cycleCount % trigger) > (trigger / 2) ) {
	    p.translate( 0, ( gEngine::PART_HEIGHT + V_GAP ) * ( trigger / 2 - (cycleCount % (trigger / 2) ) ) * 2 / trigger );
	}
	
	//pistons
	p.setBrush( C_DEVICE );
	p.drawRect(-gEngine::PART_SIZE/3, 0, gEngine::PART_SIZE*2/3, gEngine::PART_HEIGHT/3);
	p.drawRect(-gEngine::PART_SIZE/2 , gEngine::PART_HEIGHT/3, gEngine::PART_SIZE, gEngine::PART_HEIGHT/3);
	
    } else {
	p.setBrush( C_DEVICE );
	p.drawRect(-gEngine::PART_SIZE/3, 0, gEngine::PART_SIZE*2/3, gEngine::PART_HEIGHT/3);
	p.drawRect(-gEngine::PART_SIZE/2 , gEngine::PART_HEIGHT/3, gEngine::PART_SIZE, gEngine::PART_HEIGHT/3);
    }
    
    
    if (stage >= GET_ENGINE) {
	p.setWorldMatrix(preCenter);

	//untranslated coords:
	line1->setPoints( (int) x(), (int) y(), (int) holding->x(), (int) holding->y() );
	line2->setPoints( (int) x() + WIDTH, (int) y(), (int) holding->x() + gEngine::WIDTH, (int) holding->y() );
    }
    
    p.setWorldMatrix(original);
}

void gMiddle::screwPart()
{
    if (screwed) return;
    screwed = true;
    switch(whatToDo) {
		case 0:
		    holding->carcass = !defect;
		    break;
		case 1:
		    holding->valves = !defect;
		    break;
		case 2:
		    holding->starter = !defect;
		    break;
		case 3:
		    holding->lid = !defect;
		    break;
		default:
		    break;
		}
    holding->incSteps();
}

QPointArray gMiddle::areaPoints() const
{
    int dx = (int) x();
    int dy = (int) y();
    QPointArray pa(4);
    pa.putPoints(0, 4, 	0, 0, 	WIDTH, 0, 	WIDTH,HEIGHT+gEngine::PART_HEIGHT+V_GAP,	0, HEIGHT+gEngine::PART_HEIGHT+V_GAP );
    pa.translate(dx,dy);
    return pa;
}

void gMiddle::advance(int phase)
{
    if (phase == 1) {
	if (stage == NONE) return;
	cycleCount++;
	if (cycleCount % trigger == 0) {
	    //which state has finished?:
	    if (stage == GET_ENGINE) {
		makePart();
		assertArrivalCoords();
		holding->setVelocity(0, 0);
		stage = MAKE_PART;
	    } else
	    if (stage == MAKE_PART) {
		setTargetCoords();
		stage = PUT_ENGINE;

	    } else
	    if (stage == PUT_ENGINE) {
		delete line1;
		delete line2;
		toBelt->putEngine(holding);
		assertTargetCoords();
		holding->setVelocity( ((gBelt*) toBelt)->direction * xVelocity(), 0);
		holding->setZ(0);
		holding = 0;
		stage = NONE;
	    }
	}
	update();
    }
}

void gMiddle::makePart()
{
    int r = rand()%100 + 1;
    if ( r > FAILURE_CHANCE ) {
	part_color = new QColor(gEngine::C_FULL);
	defect = false;
    } else {
	part_color = new QColor(gEngine::C_BAD);
	defect = true;
    }
}

void gMiddle::setArrivalCoords()
{
    double trigger2 = trigger - 1;
    double velocity_x = (double) ( x() - holding->x() + H_GAP ) / trigger2;
    double velocity_y = (double) ( y() + HEIGHT - holding->y() + V_GAP ) / trigger2;
    holding->setVelocity(velocity_x, velocity_y);
}

void gMiddle::setTargetCoords()
{
    double trigger2 = trigger - 1;
    const QPoint *p = ((gBelt*) toBelt)->getArrivalPoint();
    double velocity_x = (double) ( p->x() - holding->x() ) / trigger2;
    double velocity_y = (double) ( p->y() - holding->y() - gEngine::HEIGHT ) / trigger2;
    holding->setVelocity(velocity_x, velocity_y);
}

void gMiddle::assertArrivalCoords()
{
    holding->setX( x() + H_GAP );
    holding->setY( y() + HEIGHT + V_GAP );
}

void gMiddle::assertTargetCoords()
{
    const QPoint *p = ((gBelt*) toBelt)->getArrivalPoint();
    holding->setX( p->x() );
    holding->setY( p->y() - gEngine::HEIGHT );
}
