#include "gEngine.h"
#include "gOmega.h"
#include <iostream>
using namespace std;

const QColor gOmega::C_DEVICE = darkCyan;
const QColor gOmega::C_INSIDE = gray;

const int gOmega::H_GAP = 10;
const int gOmega::V_GAP = 10;
const int gOmega::WIDTH = gEngine::WIDTH + 2*H_GAP;
const int gOmega::HEIGHT = gEngine::HEIGHT + 2*V_GAP;

gOmega::gOmega(QCanvas *canvas, gBelt *fromBelt):
	QCanvasPolygonalItem(canvas), Omega(fromBelt), toBeDestroyed(0)
{
    setZ(5);
}

gOmega::~gOmega()
{
    hide();
}

void gOmega::doStep()
{   
    gEngine *eng = ((gBelt*) fromBelt)->getEngineAtEnd();
    if (eng) {
	toBeDestroyed.push_back(eng);
    }
}

void gOmega::destroyOld()
{
    if (!toBeDestroyed.empty()) {
	list<gEngine*>::iterator iter;
	for (iter = toBeDestroyed.begin(); iter != toBeDestroyed.end(); iter++) {
	    //if ( qRound((*iter)->x()) == qRound(x() + H_GAP)) {
	    if ( (( ((gBelt*) fromBelt)->direction > 0 ) && ( (*iter)->x() >= x() + H_GAP ) )
	       || (( ((gBelt*) fromBelt)->direction < 0) && ( (*iter)->x() <= x() + H_GAP ) ) ) {
		total++;
		if (!(*iter)->carcass) {
		    withoutCarcass++;
		}
		if (!(*iter)->valves) {
		    withoutValves++;
		}
		if (!(*iter)->starter) {
		    withoutStarter++;
		}
		if (!(*iter)->lid) {
		    withoutLid++;
		}
		if ( ((*iter)->carcass) && ((*iter)->valves) 
		    && ((*iter)->starter) && ((*iter)->lid) )     
		    {
		       good++;
		    }
		
		emit sig_stats(total, good, withoutCarcass, withoutValves, withoutStarter, withoutLid);
			
		(*iter)->setAnimated(false);
		(*iter)->setVelocity(0, 0);
		(*iter)->hide();
		cout << "destroyed object: " << (void*) (*iter) << endl;
		toBeDestroyed.erase(iter);
		delete (*iter);
		iter--;
	    }
	}
    }
}

void gOmega::drawShape ( QPainter & p )
{
    QWMatrix original = p.worldMatrix();
    p.translate(x(), y());
    //p.setPen( NoPen );
    p.setBrush(C_DEVICE);
    p.drawRect(0, 0, WIDTH, gOmega::HEIGHT);
    
    p.setBrush(C_INSIDE);
    p.drawRect(0, V_GAP, WIDTH, HEIGHT-2*V_GAP);
    
    p.setWorldMatrix(original);
}

QPointArray gOmega::areaPoints() const
{
    int dx = (int) x();
    int dy = (int) y();
    QPointArray pa(4);
    pa.putPoints(0, 4, 	0, 0, 	WIDTH, 0, 	WIDTH,HEIGHT,	0, HEIGHT );
    pa.translate(dx,dy);
    return pa;
}

void gOmega::advance(int phase)
{
    if (phase == 0) {
	destroyOld();
    }
}
